@@ -11,6 +11,7 @@ from queue import Empty
 import signal
 import subprocess
 import sys
+from tempfile import TemporaryDirectory
 import time
 from warnings import warn
 
@@ -18,7 +19,6 @@ from typing import Dict as DictType, Any as AnyType
 
 from zmq import ZMQError
 from IPython.core import page
-from ipython_genutils.tempdir import NamedFileInTemporaryDirectory
 from traitlets import (
     Bool,
     Integer,
@@ -56,7 +56,6 @@ from prompt_toolkit.filters import (
     is_done,
 )
 from prompt_toolkit.history import InMemoryHistory
-from prompt_toolkit.patch_stdout import patch_stdout
 from prompt_toolkit.shortcuts.prompt import PromptSession
 from prompt_toolkit.shortcuts import print_formatted_text, CompleteStyle
 from prompt_toolkit.key_binding import KeyBindings
@@ -68,7 +67,7 @@ from prompt_toolkit.layout.processors import (
 from prompt_toolkit.styles import merge_styles
 from prompt_toolkit.styles.pygments import (style_from_pygments_cls,
                                             style_from_pygments_dict)
-from prompt_toolkit.formatted_text import PygmentsTokens, FormattedText
+from prompt_toolkit.formatted_text import PygmentsTokens
 from prompt_toolkit.output import ColorDepth
 from prompt_toolkit.utils import suspend_to_background_supported
 
@@ -77,6 +76,8 @@ from pygments.lexers import get_lexer_by_name
 from pygments.util import ClassNotFound
 from pygments.token import Token
 
+from jupyter_client.utils import run_sync
+
 
 def ask_yes_no(prompt, default=None, interrupt=None):
     """Asks a question and returns a boolean (y/n) answer.
@@ -160,7 +161,7 @@ class JupyterPTCompleter(Completer):
                 ):
                     new_meta[c] = m["type"]
                 meta = new_meta
-            except:
+            except Exception:
                 pass
 
         start_pos = content["cursor_start"] - document.cursor_position
@@ -662,7 +663,7 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
                     # wish to include external content
                     tasks.append(self.handle_external_iopub(loop=loop))
 
-                main_task = asyncio.wait(tasks, loop=loop, return_when=asyncio.FIRST_COMPLETED)
+                main_task = asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
                 _, pending = loop.run_until_complete(main_task)
 
                 for task in pending:
@@ -706,8 +707,8 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
             return
 
         # flush stale replies, which could have been ignored, due to missed heartbeats
-        while self.client.shell_channel.msg_ready():
-            self.client.shell_channel.get_msg()
+        while run_sync(self.client.shell_channel.msg_ready)():
+            run_sync(self.client.shell_channel.get_msg)()
         # execute takes 'hidden', which is the inverse of store_hist
         msg_id = self.client.execute(cell, not store_history)
 
@@ -740,7 +741,8 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
     #-----------------
 
     def handle_execute_reply(self, msg_id, timeout=None):
-        msg = self.client.shell_channel.get_msg(block=False, timeout=timeout)
+        kwargs = {"timeout": timeout}
+        msg = run_sync(self.client.shell_channel.get_msg)(**kwargs)
         if msg["parent_header"].get("msg_id", None) == msg_id:
 
             self.handle_iopub(msg_id)
@@ -748,8 +750,8 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
             content = msg["content"]
             status = content['status']
 
-            if status == 'aborted':
-                self.write('Aborted\n')
+            if status == "aborted":
+                sys.stdout.write("Aborted\n")
                 return
             elif status == 'ok':
                 # handle payloads
@@ -779,7 +781,8 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
         ## Get the is_complete response:
         msg = None
         try:
-            msg = self.client.shell_channel.get_msg(block=True, timeout=timeout)
+            kwargs = {"timeout": timeout}
+            msg = run_sync(self.client.shell_channel.get_msg)(**kwargs)
         except Empty:
             warn('The kernel did not respond to an is_complete_request. '
                  'Setting `use_kernel_is_complete` to False.')
@@ -835,11 +838,9 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
 
     async def handle_external_iopub(self, loop=None):
         while self.keep_running:
-            # we need to check for keep_running from time to time as
-            # we are blocking in an executor block which cannot be cancelled.
-            poll_result = await loop.run_in_executor(
-                None, self.client.iopub_channel.socket.poll, 500)
-            if(poll_result):
+            # we need to check for keep_running from time to time
+            poll_result = await self.client.iopub_channel.socket.poll(500)
+            if poll_result:
                 self.handle_iopub()
 
     def handle_iopub(self, msg_id=''):
@@ -850,10 +851,9 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
 
            It only displays output that is caused by this session.
         """
-        while self.client.iopub_channel.msg_ready():
-            sub_msg = self.client.iopub_channel.get_msg()
+        while run_sync(self.client.iopub_channel.msg_ready)():
+            sub_msg = run_sync(self.client.iopub_channel.get_msg)()
             msg_type = sub_msg['header']['msg_type']
-            parent = sub_msg["parent_header"]
 
             # Update execution_count in case it changed in another session
             if msg_type == "execute_input":
@@ -977,24 +977,22 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
         imageformat = self._imagemime[mime]
         fmt = dict(format=imageformat)
         args = [s.format(**fmt) for s in self.stream_image_handler]
-        with open(os.devnull, 'w') as devnull:
-            proc = subprocess.Popen(
-                args, stdin=subprocess.PIPE,
-                stdout=devnull, stderr=devnull)
+        with subprocess.Popen(args, stdin=subprocess.PIPE,
+                              stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL) as proc:
             proc.communicate(raw)
-        return (proc.returncode == 0)
+            return (proc.returncode == 0)
 
     def handle_image_tempfile(self, data, mime):
         raw = base64.decodebytes(data[mime].encode('ascii'))
         imageformat = self._imagemime[mime]
         filename = 'tmp.{0}'.format(imageformat)
-        with NamedFileInTemporaryDirectory(filename) as f, \
-                open(os.devnull, 'w') as devnull:
-            f.write(raw)
-            f.flush()
-            fmt = dict(file=f.name, format=imageformat)
+        with TemporaryDirectory() as tempdir:
+            fullpath = os.path.join(tempdir, filename)
+            with open(fullpath, 'wb') as f:
+                f.write(raw)
+            fmt = dict(file=fullpath, format=imageformat)
             args = [s.format(**fmt) for s in self.tempfile_image_handler]
-            rc = subprocess.call(args, stdout=devnull, stderr=devnull)
+            rc = subprocess.call(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
         return (rc == 0)
 
     def handle_image_callable(self, data, mime):
@@ -1007,7 +1005,7 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
     def handle_input_request(self, msg_id, timeout=0.1):
         """ Method to capture raw_input
         """
-        req = self.client.stdin_channel.get_msg(timeout=timeout)
+        req = run_sync(self.client.stdin_channel.get_msg)(timeout=timeout)
         # in case any iopub came while we were waiting:
         self.handle_iopub(msg_id)
         if msg_id == req["parent_header"].get("msg_id"):
@@ -1036,6 +1034,6 @@ class ZMQTerminalInteractiveShell(SingletonConfigurable):
 
             # only send stdin reply if there *was not* another request
             # or execution finished while we were reading.
-            if not (self.client.stdin_channel.msg_ready() or
-                    self.client.shell_channel.msg_ready()):
+            if not (run_sync(self.client.stdin_channel.msg_ready)() or
+                    run_sync(self.client.shell_channel.msg_ready)()):
                 self.client.input(raw_data)
